#include <math.h>
#include <cstdlib>
using namespace std;

class voice{
 public:
  voice(){
    v_delBuffer = NULL;
    v_attenuation = 0.998;
    v_maxDelay = 0;
    v_amountdelay = 0;
    v_reader = 0;
    v_writer = 0;
    v_srate = 0;
    v_lastsamp = 0;
  }

  ~voice(){
  }

 public:

  //initialize delay buffer
  bool initialize(int maxdelay, long Srate, double frequency ){
		
    v_maxDelay = maxdelay;
    v_srate = Srate;
    v_amountdelay = frequency;

    //alllocate space to delay buffer
    v_delBuffer = new double[v_maxDelay];
    
    //fill initial buffer
    for(int i = 0; i < v_maxDelay; i++){
      v_delBuffer[i] = 2*(rand() / (double)RAND_MAX) - 1;
    }

    //initial indices
    v_reader = v_writer - v_amountdelay;

    //wrap em
    if(v_reader < 0) v_reader += v_maxDelay;

    return true;
	
  }
  //actual synthesis
  void synth(double * buff, unsigned int numFrames){
	
    for(int i = 0; i < numFrames; i++){
      //read next out from delay buffer with moving average filter
      buff[i] = (v_lastsamp + v_delBuffer[v_reader]) / 2;

      //last samp equal to last samp
      v_lastsamp = buff[i];
      
      //put next input into delay buffer
      v_delBuffer[v_writer] = buff[i] * v_attenuation;
      
      //indecies...forward march!
      v_writer++;
      v_reader++;
      
      //keep them indices in line!
      v_writer %= v_maxDelay;
      v_reader %= v_maxDelay;
    }
  }

  //set the frequency 
  void setfreq(float freq){
    if(freq == 0) freq = v_freq;
    //compute delay length
    float length = v_srate / freq;

    //amount of delay
    v_amountdelay = (int) length + 0.5;
    
    //set read and write
    v_reader  = v_writer - v_amountdelay;
    
    //wrap it 
    if (v_reader < 0) v_reader += v_maxDelay;
    
  }
  
  //intial pluck, fill delay buffer with noise
  void fill(double gain){
    
    //fill buffer wih white noise
    for(int i = 0; i < v_maxDelay; i++){
      
      v_delBuffer[i] = ((2 * (rand() /  (double) RAND_MAX)) - 1) * gain;
    }
    
  }

 protected:
  //read index
  long v_reader;
  
  //write index
  long v_writer;
  
  //maximum amount of delay
  long v_maxDelay;
  
  //delay buffer
  double * v_delBuffer;
  
  //feedback attentuation
  double v_attenuation;
  
  //integer amount of delay
  int v_amountdelay;
  
  //set srate
  long v_srate;
  
  //frequency
  double v_freq;

  //variable to hold last sample
  double v_lastsamp;
  
};
